#!/usr/bin/env python
import sys
from .schema import Schema
from .schema import Data

PY2 = sys.version_info[0] == 2


def make_schema(path, parser='PyYAML', validators=None):
    # validators = None means use default.
    # Import readers here so we can get version information in setup.py.
    from . import readers
    raw_schemas = readers.parse_file(path, parser)

    # First document is the base schema
    try:
        s = Schema(raw_schemas[0], path, validators=validators)
        # Additional documents contain Includes.
        for raw_schema in raw_schemas[1:]:
            s.add_include(raw_schema)
    except (TypeError, SyntaxError) as e:
        error = 'Schema error in file %s\n' % path
        error += str(e)
        if PY2:
            error.encode('utf-8')
        raise SyntaxError(error)

    return s


def make_data(path, parser='PyYAML'):
    from . import readers
    raw_data = readers.parse_file(path, parser)
    return [Data(d, path) for d in raw_data]


def validate(schema, data):
    for d in data:
        schema.validate(d)
    return data



import os

import pytest

from .. import command_line

dir_path = os.path.dirname(os.path.realpath(__file__))

parsers = ['pyyaml', 'PyYAML', 'ruamel']


@pytest.mark.parametrize('parser', parsers)
def test_bad_yaml(parser):
    try:
        command_line._router(
            'yamale/tests/command_line_fixtures/yamls/bad.yaml',
            'schema.yaml', 1, parser)
    except ValueError as e:
        assert 'Validation failed!' in str(e)
        return
    assert False


@pytest.mark.parametrize('parser', parsers)
def test_good_yaml(parser):
    command_line._router(
        'yamale/tests/command_line_fixtures/yamls/good.yaml',
        'schema.yaml', 1, parser)


@pytest.mark.parametrize('parser', parsers)
def test_good_relative_yaml(parser):
    command_line._router(
        'yamale/tests/command_line_fixtures/yamls/good.yaml',
        '../schema_dir/external.yaml', 1, parser)


@pytest.mark.parametrize('parser', parsers)
def test_external_glob_schema(parser):
    command_line._router(
        'yamale/tests/command_line_fixtures/yamls/good.yaml',
        os.path.join(dir_path, 'command_line_fixtures/schema_dir/ex*.yaml'), 1, parser)


def test_external_schema():
    command_line._router(
        'yamale/tests/command_line_fixtures/yamls/good.yaml',
        os.path.join(dir_path, 'command_line_fixtures/schema_dir/external.yaml'), 1, 'PyYAML')


def test_bad_dir():
    try:
        command_line._router(
            'yamale/tests/command_line_fixtures/yamls',
            'schema.yaml', 4, 'PyYAML')
    except ValueError as e:
        assert 'Validation failed!' in str(e)
        return
    assert False



import pytest
from .. import yaml_reader
from yamale.tests import get_fixture

parsers = ['pyyaml', 'PyYAML', 'ruamel']
TYPES = get_fixture('types.yaml')
NESTED = get_fixture('nested.yaml')
KEYWORDS = get_fixture('keywords.yaml')


@pytest.mark.parametrize('parser', parsers)
def test_parse(parser):
    a = yaml_reader.parse_file(TYPES, parser)[0]
    assert a['string'] == 'str()'


@pytest.mark.parametrize('parser', parsers)
def test_types(parser):
    t = yaml_reader.parse_file(TYPES, parser)[0]
    assert t['string'] == 'str()'
    assert t['number'] == 'num()'
    assert t['boolean'] == 'bool()'
    assert t['integer'] == 'int()'


@pytest.mark.parametrize('parser', parsers)
def test_keywords(parser):
    t = yaml_reader.parse_file(KEYWORDS, parser)[0]
    assert t['optional_min'] == 'int(min=1, required=False)'


@pytest.mark.parametrize('parser', parsers)
def test_nested(parser):
    t = yaml_reader.parse_file(NESTED, parser)[0]
    assert t['list'][-1]['string'] == 'str()'



from __future__ import absolute_import


def _pyyaml(file_name):
    import yaml
    try:
        Loader = yaml.CSafeLoader
    except AttributeError:  # System does not have libyaml
        Loader = yaml.SafeLoader
    with open(file_name) as f:
        return list(yaml.load_all(f, Loader=Loader))


def _ruamel(file_name):
    from ruamel.yaml import YAML
    yaml = YAML(typ='safe')
    with open(file_name) as f:
        return list(yaml.load_all(f))


_parsers = {
    'pyyaml': _pyyaml,
    'ruamel': _ruamel
}


def parse_file(file_name, parser):
    try:
        parse = _parsers[parser.lower()]
    except KeyError:
        raise NameError('Parser "' + parser + '" is not supported\nAvailable parsers are listed below:\nPyYAML\nruamel')
    return parse(file_name)
